using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

namespace HyttysPeli;

/// @author salmatel
/// @version 12.11.2023
/// <summary>
/// 
/// </summary>
public class HyttysPeli : PhysicsGame
{
    public override void Begin()
    {
        MasterVolume = 0.05;
        Level.Background.Color = Color.Green;
        BoundingRectangle alaosa = new BoundingRectangle(new Vector(Level.Left, 0), Level.BoundingRect.BottomRight);
        BoundingRectangle ylaosa = new BoundingRectangle(Level.BoundingRect.TopLeft, new Vector(Level.Right, 0));
        
        Level.CreateBorders();
        
        PhysicsObject pelaaja = new PhysicsObject(100, 100, Shape.Rectangle);
        pelaaja.Position = RandomGen.NextVector(alaosa);
        pelaaja.Image = LoadImage("hyttyskarkoite");
        Add(pelaaja);
        
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä avustus");
        Keyboard.Listen(Key.Up, ButtonState.Down, LyoHyttysta, "Pelaaja ylös", pelaaja, new Vector(0, 20));
        Keyboard.Listen(Key.Down, ButtonState.Down, LyoHyttysta, "Pelaaja alas", pelaaja, new Vector(0, -20));
        Keyboard.Listen(Key.Left, ButtonState.Down, LyoHyttysta, "Pelaaja vasemmalle", pelaaja, new Vector(-20, 0));
        Keyboard.Listen(Key.Right, ButtonState.Down, LyoHyttysta, "Pelaaja oikealle", pelaaja, new Vector(20, 0));


        for(int i = 0; i < 20; i++) 
            LuoSattunnainenHyttynen(this, ylaosa, 60, "hyttynen");
        

        AddCollisionHandler(pelaaja, "hyttynen", PelaajaTormasi);

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    
    /// <summary>
    /// Metodi johon tullaan kun pelaaja ja joku hyttynen törmäävät.
    /// Luodaan hyttysen kohdalla räjähdys ja poistetaan hyttynen pelistä
    /// </summary>
    /// <param name="pelaaja">pelaaja joka törmäsi</param>
    /// <param name="hyttynen">hyttynen johon pelaaja törmäsi</param>
    private void PelaajaTormasi(PhysicsObject pelaaja, PhysicsObject hyttynen)
    {
        double pelaajanAla = HyttysenAla(pelaaja.Width, pelaaja.Height);
        double hyttysenAla = HyttysenAla(hyttynen.Width, hyttynen.Height);

        if (pelaajanAla > hyttysenAla)
        {
            Explosion rajahdys = new Explosion(hyttynen.Width * 2);
            rajahdys.Position = hyttynen.Position;
            rajahdys.UseShockWave = false; // kannattaa kokeilla mitä taphatuu true -arvolla. 
            this.Add(rajahdys);
            Remove(hyttynen);
        }
    }
    /// <summary>
    /// Lyödään hyttystä voimavektorilla
    /// </summary>
    /// <param name="hyttynen">lyötävä hyttynen</param>
    /// <param name="suunta">voimavektori</param>
    public static void LyoHyttysta(PhysicsObject hyttynen, Vector suunta)
    { 
        hyttynen.Hit(hyttynen.Mass * suunta);
    }

    
    /// <summary>
    /// Luodaan satunnainen hyttynen suorakaiteen sisälle.
    /// </summary>
    /// <param name="peli">Mihin peliin hyttynen luodaan</param>
    /// <param name="rect">Minkä suorakaiteen sisälle luodaan</param>
    /// <param name="vauhti">Hyttysen vauhti alkutilanteessa</param>
    /// <param name="tunniste">Hyttyselle annettava tunniste</param>
    /// <returns>Viite luotuun hyttyseen</returns>
    public static PhysicsObject LuoSattunnainenHyttynen(PhysicsGame peli, BoundingRectangle rect, double vauhti, string tunniste)
    {
        double leveys = 50;
        double korkeus = 50;
        PhysicsObject hyttynen = new PhysicsObject(leveys, korkeus, Shape.Circle);
        hyttynen.Position = RandomGen.NextVector(rect);
        Vector suunta = RandomGen.NextVector(0, vauhti);
        hyttynen.Hit(suunta);
        hyttynen.Tag = tunniste;
        hyttynen.Image = LoadImage("hyttynen (2)");
        peli.Add(hyttynen);
        return hyttynen;
    }
    


    /// <summary>
    /// Laskee hyttysen alan
    /// </summary>
    /// <param name="kanta">hyttysen kanta</param>
    /// <param name="korkeus">hyttysen korkeus</param>
    /// <returns>hyttysen ala</returns>
    /// <example>
    /// <pre name="test">
    ///    HyttysenAla(0,0) ~~~ 0;
    ///    HyttysenAla(1,0) ~~~ 0;
    ///    HyttysenAla(1,1) ~~~ 0.5;
    ///    HyttysenAla(3,5) ~~~ 7.5;
    /// </pre>
    /// </example>
    public static double HyttysenAla(double kanta, double korkeus)
    {
        return kanta * korkeus * 1 / 2;
    }
}
